package dmc;

/* 
 * DMC.java
 *
 * Matthew Wronka
 * Last Modified 2002-11-25
 *
 *
 * Main shell for messaging client
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 *
 */


import javax.swing.*;
import javax.swing.text.*;

import java.awt.*;
import java.awt.event.*;

import java.net.*;

import java.io.IOException;
import java.util.StringTokenizer;

/**
 * This is the main shell for the messaging client.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 *
 */
public class DMC {
	final public static String APP_VERSION = "20031129";
	final public static String APP_NAME = "Distributed Messaging Client";
	final public static String DEFAULT_CONNECTION = "DMC";
	final public static int DEFAULT_PORT = 5050;


	/**
	 * Returns a static string indicating application name an version.
	 * 
	 * @return Returns a static string indicating application name an version.
	 */
	static public String getVersion(){
		return APP_NAME + " version " + APP_VERSION;
	}

	/**
	 * Prints the current version as returned by <code>String getVersion()</code>
	 */
	static public void printBanner(){
		System.out.println(getVersion());
	}

	/**
	 * Loads the initial script for startup defaults.
	 */
	static private void loadDmcrc(GUI myGUI) {
               // Run startup scripts
                myGUI.getTabbedConnection().getTabbedChat(DEFAULT_CONNECTION).
                                getInputPanel(null).runScript (
					System.getProperty("user.home") +
					System.getProperty("file.separator") +
					".dmc" +
					System.getProperty("file.separator") +
					"dmcrc"
				);
	}


	/**
	 * Prints version string and creates GUI.
	 */
	public static void main(String[] args){
		printBanner();
		GUI myGUI = null;

		wtalk.WTalk.setHostClient(APP_NAME);

		for(int i=0; i < args.length; ++i) {
			if(args[i].charAt(0) == '-')
				i = DmcSettings.handleArg((args[i]).substring(1), args, i);
		}

		//if(args.length >0) 
		//	myGUI = new GUI(args[0]);
		//else
			myGUI = new GUI();

		DmcServer.setTabbedConnection(myGUI.getTabbedConnection());
		if(DmcSettings.bServer) DmcServer.CreateSocket(0);
		if(DmcSettings.bLoadDmcrc) loadDmcrc(myGUI);

	}
}


class DmcSettings {
	static public boolean bLoadDmcrc = true;
	static public boolean bServer = true;
	static public int iServerPort = 0;
	static public boolean bGUI = true;


	/**
	 * Handles command line arguments and switches.
	 */
	static  public int handleArg(String arg, String[] args, int index) {
		if(arg.equals("h") || arg.equals("help")) {
			System.err.println("java dmc/DMC " +
				" [-h|-nodmcrc|-noserver|-nogui]*");
			System.err.println
				("\t-h\t\tPrincts this message and exits.");
			System.err.println
				("\t-nodmcrc\tDoes not load startup script.");
			System.err.println
				("\t-noserver\tDoes not start DMC server.");
			System.err.println
				("\t-nogui\tDoes not show DMC GUI.");
			System.exit(1);
		}
		if(arg.equals("nodmcrc")) { 
			bLoadDmcrc = false;
			return index;
		}
		if(arg.equals("noserver")) { bServer = false; return index; }
		if(arg.equals("port")) { ; return index; }
		if(arg.equals("nogui")) { bGUI = false; return index; }

		return index;
	 }
}
