package dmc;

/*
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;

import java.awt.*;
import java.awt.event.*;

import java.net.*;

import java.io.IOException;
import java.util.StringTokenizer;

 /**
  * GUI for the DMC messaging system. This creates and organizes
  *	all fo the necesary GUI components.  This must be called
  *	with a wrapper instead of being run directly.
  *
  * @author Matthew Wronka <wronkm@rpi.edu>
  * @version 0
  * @see DMC
  */
public class GUI {
	final protected TabbedConnection tabbedConnection; 
	final protected UserList userList = new UserList();

	/**
	 * Default ctor.
	 */
	public GUI() {
		tabbedConnection = new TabbedConnection();
		initGUI();
	}
	/**
	 * Overloaded ctor.
	 *
	 * @param DefaultProtocol Use  <em>DefaultProtocol</em> instead of
	 *	compile time default.
	 */
	public GUI(String DefaultProtocol) {
		tabbedConnection = new TabbedConnection(DefaultProtocol);
		initGUI();
	}

	/**
	 * Initializes the GUI class.
	 */
	private void initGUI(){
		final JFrame frame = new JFrame(DMC.getVersion());

		try {
			UIManager.setLookAndFeel(
				UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {}


		initComponents();


		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(createComponents(), BorderLayout.CENTER);
		frame.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				System.exit(0);
			}
		});

		frame.pack();
		frame.setSize (480, 320);
		frame.show();

	}

	/**
	 * Initializes the components.
	 */
	protected void initComponents() {
		tabbedConnection.setUserList(userList);

		userList.registerMessageGUI(tabbedConnection);
	}

	/**
	 * Returns a pane containing all the necessary components of the GUI.
	 *
	 * @return A pane whith all components used by the GUI.
	 */
	protected Component createComponents() {
		final JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());

		pane.add(tabbedConnection.getUI(), BorderLayout.CENTER);
		pane.add(userList.getUI(), BorderLayout.EAST);

		return pane;

	}


        /**
         * Returns the tabbed connection object which manages conversations.
         *
         * @return TabbedConnection object
         */
	public TabbedConnection getTabbedConnection(){return tabbedConnection;}

}
