package jsh;

/* jsh.java
 * Matthew Wronka
 *
 * Command Shell similar to the bourne family.
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */


import java.io.*;

import java.lang.*;
import java.util.StringTokenizer;

/**
 * 
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
public class jsh{
	protected jsh_process m_jp;
        protected InputStream m_is = System.in;
        protected PrintStream m_os = System.out;
	protected PrintStream m_es = System.err;
	protected OutputTarget m_ot = null;
	protected OutputTarget m_et = null;


	/**
	 * Bournesque shell
	 * The main function is a wrapper for other functions.
	 * It must first parse passed arguments, sets run mode,
	 * and then invokes the main control loop (normally).
	 *
	 */
	 public static void main(String[] args) {
	 	jsh my_jsh = new jsh();
		my_jsh.jsh_loop(args);
	 }

	/**
	 * Main loop. Read Execute Loop.
	 */
	 public void jsh_loop(String[] args) {
	 	String sCommand="";
		String sPrompt="> ";
		BufferedReader br = new BufferedReader(
					new InputStreamReader(System.in)
				);


		do {
			System.out.print(sPrompt);
			try {
				sCommand = br.readLine();
			} catch (IOException ioe) { }

			if(sCommand == null) {
				// EOF
				break;
			}

		//System.out.println("Bottom of Loop.\n");
		}while(jsh_execute(sCommand.trim()));
	 }

	/**
	 * This function must be called if one wishes to manipulate the process,
	 * or to get a stream directly from the process.
	 *
	 * @param sCommand Command to execute.
	 */
	public void setupProcess(String sCommand) {
		if(m_os != null)
			m_jp=new jsh_process(tokenize(sCommand), m_is, m_os, m_es);
		else if(m_ot != null)
			m_jp=new jsh_process(tokenize(sCommand), m_is, m_ot, m_et);
		else
			m_jp = new jsh_process(tokenize(sCommand), m_is, m_ot=null, null);
	}

	  /**
	   * Executes the givven command, returning false if the command
	   * would terminate the shell.
	   * <em>Note:</em> This <strong>DOES NOT</strong> return the
	   * exit code of the appliation run.
	   *
	   * @param sCommand Command to execute (eg. as typed into the shell).
	   *	sCommand should be 'null' if one is using setupProcess.
	   * @return Returns true when the command is execute (regardless of
	   * 	the commands exit code), or false if the command terminates
	   *	the shell.
	   */
	   public boolean jsh_execute(String sCommand) {
		if(sCommand.equals("logout") || sCommand.equals("exit"))
				return false;

		if(sCommand!=null) setupProcess(sCommand);

		m_jp.exec();
		//System.out.println("End o' Exec\n");
		return true;
	   }

	/**
	 * Sets the stream that executed programs see as stdin.
	 *
	 * @param es New target for stdin.  This is used by executed programs.
	 */
	 public void jsh_setIn(InputStream is) { m_is = is; }

	/**
	 * Sets the stream that executed programs see as stdout.
	 *
	 * @param es New target for stdout.  This is used by executed programs.
	 */
	 public void jsh_setOut(PrintStream os) { m_os = os; }
	 public void jsh_setOut(OutputTarget ot) { m_ot = ot; }


	/**
	 * Returns the an InputStream from process' stdout.
	 *
	 * @return STDOUT of executing process.
	 */
	public InputStream getOutputStream() {
		return m_jp.getOutputStream();
	}


	/**
	 * Sets the stream that executed programs see as stderr.
	 *
	 * @param es New target for stderr.  This is used by executed programs.
	 */
	 public void jsh_setErr(PrintStream es) { m_es = es; }
	 public void jsh_setErr(OutputTarget et) { m_et = et; }

	/**
	 * Tokenizes a string, taking into account double quotes.
	 * XXX: clarify
	 *
	 * @param str Command with arguments in a single string.
	 * @return Array of strings with the cmd as element 0.
	 */
	 static public String[] tokenize(String str) {
	 	String[] rval = null;
		java.util.Vector vrval = new java.util.Vector();
		String currStr = "";
		boolean dblquote=false;

		// This may get more complated later, to a point
		// where StringTokenizer will simply not work --
		// at least not without a lot of ugliness . . .
		for(int i=0; i<str.length(); ++i) {


			switch(str.charAt(i)) {
			case ' ':
				if(!dblquote && i!=0 && str.charAt(i-1)!=' ') {
					vrval.add(currStr);
					currStr="";
					while(i<str.length() && 
						str.charAt(i)==' ') ++i;

					if(i<str.length()) --i;
					break;
				}
				currStr += str.charAt(i);
				break;

			case '"':
				dblquote = !dblquote;
				break;
			case '\\':
				++i;
				if(i >= str.length()) {
					System.err.println("Illformed (trailing '\\')\n");
					return null;
				}
			default:
				currStr += str.charAt(i);
			}
		}

		if(dblquote) {
			//System.err.println("Illformed (unmatched '\"')\n");
			return null;
		}

		vrval.add(new java.lang.String(currStr));

		// copy vrval to val
		rval = new String[vrval.size()];
		for(int i=0; i<vrval.size(); ++i) {
			rval[i]=(String)vrval.get(i);
		}

		return (rval);
	 }


}


class jsh_process {
	private String[] m_toexec = null;
	protected InputStream m_is = null;
	protected PrintStream m_os = null;
	protected PrintStream m_es = null;
	protected OutputTarget m_ot = null;
	protected OutputTarget m_et = null;
	protected boolean m_bBackground = false;

	protected Process m_proc = null;

	public jsh_process(String[] sa, InputStream is, PrintStream os, 
				PrintStream es) {
		m_os = os; m_is = is; m_es = es;
		m_toexec = sa;

		try {
			m_proc = java.lang.Runtime.getRuntime().exec(m_toexec);
		} catch (java.io.IOException ioe) {}
	}
	
	public jsh_process(String[] sa, InputStream is, OutputTarget ot, 
				OutputTarget et) {
		m_ot = ot; m_is = is; m_et = et;
		m_toexec = sa;
		try {
			m_proc = java.lang.Runtime.getRuntime().exec(m_toexec);
		} catch (java.io.IOException ioe) {}
	}

	public void setBackground(boolean bBackground) {
		m_bBackground = bBackground;
	}

	public InputStream getOutputStream() {
		return m_proc.getInputStream();
	}

	public boolean exec() {
	   	Process p=m_proc; 
		PipedOutputStream POS; 
		PipedInputStream PIS;

		jsh_piper jpIn=null;
		jsh_piper jpOut=null;
		jsh_piper jpErr=null;

		try {


			if(m_os!=null) {
				// Read from
				jpIn = new jsh_piper(p.getInputStream(), m_os);
				jpErr = new jsh_piper(p.getErrorStream(), m_es);
				// Write to 
				jpOut = new jsh_piper(m_is, 
					new PrintStream(p.getOutputStream()));
			} else if (m_ot != null) {
				jpIn = new jsh_piper(p.getInputStream(), m_ot);
				jpErr = new jsh_piper(p.getErrorStream(), m_et);
				jpOut = new jsh_piper(m_is, 
					new PrintStream(p.getOutputStream()));
			}

			//System.out.println("Launching pide-pipers.\n");
			//jpOut.start(); 
			if(jpIn != null) jpIn.start(); 
			if(jpErr !=null) jpErr.start();
			//System.out.println("Going to wait.\n");
			p.waitFor();
			// This line is deprecated.  Oh well.
			// Bigger concern is lost buffered data?
			//jpOut.stop();
			//jpIn.stop(); jpOut.stop(); jpErr.stop();
			//System.out.println("About to return.\n");
			return true;
		//} catch ( IOException ioe) {
			//System.err.println("IO Exception caught.\n");
		} catch (InterruptedException ie) {
			//System.err.println("Int Exception caught.\n");
		}

		return false;
	}
}

class jsh_piper extends Thread {
	protected InputStream m_in = null;
	protected PrintStream m_out = null;
	protected OutputTarget m_ot = null;

	public jsh_piper(InputStream in, PrintStream out) {
		m_in = in; m_out = out;
	}

	public jsh_piper(InputStream in, OutputTarget ot) {
		m_in = in; m_ot = ot;
	}

	public void destroy() { return;  }

	public void run() {
		final int buffersize = 1024;
		byte[] b = new byte[buffersize];
		int toread = buffersize;
		int read = 0;

		do {
			read = 0;
			try {
				toread = m_in.available();
				if(toread == 0 ||toread > buffersize) 
					toread = buffersize;

				read = m_in.read(b, 0, toread);

				// This can be generalized later...
				if(m_out != null)
					m_out.write(b, 0, read);
				else
					m_ot.write(new String(b,0,read));

			} catch (Exception e){
				//System.err.println("Broken pipe?\n");
				break;
			}
		} while(read >= 0) ;
		// System.err.println("Out of while.\n");
	}
}
