package wtalk.protocols.irc;

import java.io.*;
import java.net.*;


import wtalk.RMsgHandler;

class ReceivedListener extends Thread{
	private IRC parent=null;
	private BufferedReader in=null;
	private IRCProtocol mIRC=null;
	private RMsgHandler rmh=null;	// generic message handler
	private RMsgHandler rwh = null;	// handles wall messages if !null
	private RMsgHandler rsh = null;	// handles system messages if !null
	private RMsgHandler rph = null;	// handles private messages if !null
	private RMsgHandler rah = null;	// handles automated messages if !null
	private RMsgHandler dmch = null;// handles client messages if !null
	private StatusListener sl = null; //listens for status resposnes

	//protected final HashMap users = new HashMap();	// Maps user => channel

	public ReceivedListener(IRC par){
		parent = par;
		in = null; rmh = null; mIRC=null;
	}

	public void ListenTo(BufferedReader n){
		in = n;
	}

	public void ReportTo(RMsgHandler r){
		rmh = r;
	}

	public void ReportWallTo(RMsgHandler r){
		rwh = r;
	}

	public void ReportSystemTo(RMsgHandler r){
		rsh = r;
	}

	public void ReportPrivateTo(RMsgHandler r){
		rph = r;
	}

	public void ReportAutoTo(RMsgHandler r){
		rah = r;
	}

	public void ReportClientTo(RMsgHandler r){
		dmch = r;
	}

	public void setStatusListener(StatusListener s){
		sl = s;
	}

	public void setIRCProtocol(IRCProtocol irc) { mIRC = irc; };


	public void CTCPHandle(String command, String target, 
			String from, String data, boolean bOriginator,
			boolean bPrivMsg) {


		// Is it something we want?
		if(! command.equalsIgnoreCase("ACTION") && bOriginator) return;

		// Extended Data
		if(command.equalsIgnoreCase("ACTION")) {
			if(target.charAt(0)=='#' || target.charAt(0)=='&'
				|| from.equals(parent.getName()) ) 
			rsh.handle(target, from, "***" + from + " " + data);
			else
			rsh.handle(from, from, "***" + from + " " + data);
			// Just make sure it's the right conversation

			return;
		} else if(command.equalsIgnoreCase("DCC")) {
		} else if(command.equalsIgnoreCase("SED")) {


		// Request/Reply
		} else if(command.equalsIgnoreCase("VERSION")) {
			if(bPrivMsg) {
				mIRC.sendCTCP(from, "VERSION", 
						wtalk.WTalk.getName() 
						+ ", " +
						IRC.getBanner()
						+ " : " +
						wtalk.WTalk.getVersion()
						+ " : " +
						wtalk.WTalk.getOperatingEnvironment()
							, true);
			} else {
				rsh.handle(target, from, "***" + from + 
						" uses " + data);

			}

			return;
		} else if(command.equalsIgnoreCase("SOURCE")) {
			if(bPrivMsg) {
				mIRC.sendCTCP(from, "SOURCE",
						wtalk.WTalk.getSourceLocation(),
						true);
			}
			return;
		} else if(command.equalsIgnoreCase("USERINFO")) {
		} else if(command.equalsIgnoreCase("CLIENTINFO")) {
		} else if(command.equalsIgnoreCase("ERRMSG")) {
		} else if(command.equalsIgnoreCase("PING")) {
		} else if(command.equalsIgnoreCase("TIME")) {
		} else {

		// Pass it back to the client?
		// It's a dmc Command
		
		// It's some unknown Command
		// Reverse from and target to make show in the window
		 if(dmch != null) {
			dmch.handle(target, from, command + data);
			System.err.println("dmch.handle("+target+","+from+", "+
				command + data + ");\n");
			}
		/* "CTCP " + command + " from " + from + ": " + data);	*/

		}
	}

	public void run(){
		String s;
		String target, from, data;
		IRCMessage message=null;

		if(in==null) return;

		while(true){

			try{
				s = in.readLine();
			} catch(IOException e){ return; }


			if(s==null) break;

			message = new IRCMessage(s);
			target = message.getReceiver();
			from = message.getSender();
			data = message.getText();

			// Message was already handled, ignore 
			if(sl!=null && 
				sl.handledStatusMessage(message)) 
				continue;

			if(message.isServerMsg() ) {

				if(message.getCommand().equals("PING")){
					mIRC.sendPong(from, data);
				} else
					rsh.handle(target, from, data);
				// System.out.println("debug: "+ s);
			} else if(message.isCTCP()) {
				System.err.println("Handling CTCP");
				CTCPHandle(message.getCTCPCommand(),
						target, from, data, false, 
						message.getCommand().equalsIgnoreCase("PRIVMSG"));
			} else if(message.isAutomatedMsg()) {
				if ( rah != null) {
					rah.handle(target, target, data);
				}
			} else if(message.getCommand().equals("QUIT")){
				// Send QUIT to each channgel user is in

				// Otherwise just send to System pannel
				rsh.handle(from, from, message.toString());
			} else if(message.isStatusMsg()
				|| (from==null) || (from.length()==0) ){


				if(target!=null && target.equalsIgnoreCase(
					parent.getName() ))
					target=null;

				if ( (from==null) || (from.length()==0) )
					target=null;


				rsh.handle(target, from, message.toString());
			
			}else if(target != null && target.length() > 0) {
				System.err.println("Regular message?");
				// When we join a ROOM, update users hash
				// Check to see if message is a JOIN or NICK

				// If  it's a JOIN, update users hash

				// If it's a NICK, change the key in the hash


				// Continue as normal
				if( !(target.charAt(0)=='&' || 
					target.charAt(0)=='#')) {
					rph.handle(target, from, data);
				} else {
					rmh.handle(target, from, data);
				}
			}

			
		}

		in=null;

	}
}

