package wtalk.protocols.toc;

import java.io.*;
import java.net.*;
import java.util.*;

import wtalk.RMsgHandler;
import wtalk.IUserList;
import wtalk.protocols.WTalkProtocol;
import wtalk.protocols.toc.jaim.*;

public class Toc implements WTalkProtocol {
	final static int DEFAULT_PORT = 9898;
	final static String DEFAULT_HOST = "toc.oscar.aol.com";
	final static int DEFAULT_TIMEOUT = 30000;

	final static String VERSION = "20031129";
	final static String NAME = "WTalk JAIM interface";
	final static String AUTHOR = "Matthew Wronka <wronkm@cs.rpi.edu>";

	protected JaimConnection jc = null;
	protected EventListener ev = null;
	//protected Socket us = null;
	//protected  PrintWriter out = null;
	//private ReceivedListener rl = null;
	protected IUserList mUL = null;
	protected UserListMaintainer mULM = null;

	private String mConnectionName = "TOC";
	private String mServer=	DEFAULT_HOST;
	private String mName="";
	private String mPassword="";
	private int mPort=	DEFAULT_PORT;

	// handles displaying of messages if !null, and not overridden;
	private RMsgHandler dmh = null;  
	// handles displaying of private messages if !null;
	private RMsgHandler sph = null;  
	// handles displaying of system messages if !null;
	private RMsgHandler smh = null;  
	// handles displaying of system messages if !null;
	private RMsgHandler rah  = null;  
	// handles DMC specific messages if !null;
	private RMsgHandler dmch = null;

	static public String getBanner() {
		return NAME + " v" + VERSION + " by " + AUTHOR;
	}

	public Toc(String ConnectionName) {
		mConnectionName = ConnectionName;
		System.err.println(getBanner());
		TocInit();
	}

	private void TocInit() {
		//rl = new ReceivedListener(this);
		//sl = new StatusListener(this);
		//rl.setStatusListener(sl);
	}

	private void resetReader() {
		//rl.ReportTo(dmh);
		//rl.ReportPrivateTo(sph);
		//rl.ReportSystemTo(smh);
		//rl.ReportAutoTo(rah);
		//rl.ReportClientTo(dmch);
		ev = new EventListener( getName(), mULM, dmch, rah, smh, dmh);
	}


	/***
	 *	Below we have the implementation of the
	 *	WTalkProtocol interface.
	 ***/

	//

	// Callbacks
	//
	public void setReceivedMessageHandler(RMsgHandler r) { 
		//rl.ReportTo(r); 
		dmh=r;
	}	// Default
	public void setChatMessageHandler(RMsgHandler r) { 
		//rl.ReportTo(r); 
	}	// Chats
	public void setPrivateMessageHandler(RMsgHandler r) { 
		//rl.ReportPrivateTo(r); 
		sph=r;
	} // Private
	public void setWallMessageHandler(RMsgHandler r) { 
		//rl.ReportWallTo(r); 
	}	// Walls
	public void setSystemMessageHandler(RMsgHandler r) { 
		//rl.ReportSystemTo(r); 
		smh=r;
	}	// System..

	public void setAutoMessageHandler(RMsgHandler r){ 
		//rl.ReportAutoTo(r); 
		rah = r;
	}

	public void setClientMessageHandler(RMsgHandler r){
		//rl.ReportClientTo(r);
		dmch = r;
	}

	public void setUserListHandler(wtalk.IUserList ul){
		// TODO:  Implement user lsit
		mUL = ul;
		if(mUL != null)
			mULM = new UserListMaintainer( mConnectionName, mUL);
		else
			mULM = null;
	} 

	//
	// Uncooked functions
	//
	public void sendRaw(String msg) { 
		// XXX: Send raw messages?
	}


	//
	// Authentication/Login
	//
	public void connect() throws IOException {
		TocInit();
		resetReader();

		jc = new JaimConnection(mServer, mPort==0? DEFAULT_PORT : mPort);		
		jc.addEventListener(ev);
		jc.setDebug(true);
		try {
			jc.connect();
			addBuddy("thewronk","Client Designer");
			jc.logIn(mName, mPassword, DEFAULT_TIMEOUT);
		} catch (JaimException je) { /* TODO: Handle this? */  }

	}

	public void connect(String login) throws IOException {
		connect(mServer, mPort, login);
	}

	public void connect(String server, int port) throws IOException {
		mServer = server;	mPort = port;
		connect();
	}

	public void connect(String server, int port, String login) throws IOException {
		mName = login;
		connect(server, port);
		// send login;
	}

	public void disconnect(String msg) { 
		if(jc!=null) {
			jc.logOut();

			try {
				jc.disconnect();
			} catch (IOException ioe) { /* TODO: Handle this? */  }
		}
	}


	public boolean isConnected() { 
		return jc!=null && jc.isLoginComplete();
	}

	public void setName(String name) { 
		if(!isConnected()) 
			mName = name; 
	}

	public String getName() { 
		return jc==null?mName:jc.getNickName(); 
	}

	public void setPassword(String passwd) { mPassword = passwd; }
	public void changePassword(String oldpwd, String newpwd) {  }

	public void setServer(String server) { 
		mServer = server.equals("")? DEFAULT_HOST : server; 
	}

	public String getServer() { return mServer; }

	public void setServerPort(int port) { 
		mPort = (port > 0 ) ? port : DEFAULT_PORT;
	}
	public int getServerPort() { return mPort; }



	//
	// IM functions
	//
	public void sendMessage(String to, String data) {
		try {
			jc.sendIM(to, data);
		} catch (IOException ioe) { /* TODO: Handle this? */  }
		dmh.handle(to, getName(), data);
	}

	public void whois(String user) { 
		try {
			jc.getInfo(user);
		} catch (IOException ioe) { /* TODO: Handle this? */  }
	}

	public void addBuddy(String buddy, String group){
		try {
			jc.watchBuddy(buddy);
		} catch (JaimException je) { /* TODO: Handle this? */  }
	}
	public void removeBuddy(String buddy, String group){
		jc.unwatchBuddy(buddy);
	}
	//listBuddies();

	public void setAway(String message){
		if(message==null) message="";
		try{
			jc.setAway(message);
		} catch (IOException ioe) { /* TODO: Handle this? */  }
	}

	//addDeny();
	//removeDeny();


	//
	// Chat functions
	//
	public void joinChat(String room) { 
	}

	public void partChat(String room) { 
	}

	public void chatMessage(String room, String data) { 
	}

	public void setTopic(String room, String topic) {
	}

	public void chatList(String room) {
	}


	//
	// Admin functions
	//
	public void sendWall(String msg) {	}

	public void sendSub(String to, String data){
		// TODO: Send <!-- Sub -->
		String subData = "<!--" + data + "-->";
		try {
			jc.sendIM(to, subData); 
		} catch (IOException ioe) { /* TODO: Handle this? */  }
	}

	public void setBan(String user, boolean banned){}
}
