package wtalk.protocols.usa;

import java.io.*;
import java.net.*;
import java.util.*;

import wtalk.RMsgHandler;
import wtalk.protocols.WTalkProtocol;

public class Usa implements WTalkProtocol {
	final static int DEFAULT_PORT = 5050;

	final static String VERSION = "20021111";
	final static String NAME = "Secure Multi-Party Chat Environment";
	final static String AUTHOR = "Matthew Wronka <wronkm@rpi.edu>";

	protected Socket us = null;
	protected  PrintWriter out = null;
	protected StatusListener sl = null;
	private ReceivedListener rl = null;

	private String mServer="";
	private String mName="";
	private int mPort=0;

	// handles sending of messages if !null, and not overridden;
	private RMsgHandler dmh = null;  
	// handles sending of private messages if !null;
	private RMsgHandler sph = null;  
	// handles sending of system messages if !null;
	private RMsgHandler smh = null;  

	static public String getBanner() {
		return NAME + " v" + VERSION + " by " + AUTHOR;
	}

	public Usa(String ConnectionName) {
		System.err.println(getBanner());
		UsaInit();
	}

	private void UsaInit() {
		rl = new ReceivedListener();
		sl = new StatusListener(this);
		rl.setStatusListener(sl);
	}

	private void resetReader() {
		rl.ReportTo(dmh);
		rl.ReportPrivateTo(sph);
		rl.ReportSystemTo(smh);
	}


	public void nickSuccess(String nick){
		mName=nick;
		RMsgHandler r = smh!=null?smh:dmh;
		if(r!=null)
			r.handle(mName,"*", "You are now known as " + nick);
	}


	/***
	 *	Below we have the implementation of the
	 *	WTalkProtocol interface.
	 ***/

	//

	// Callbacks
	//
	public void setReceivedMessageHandler(RMsgHandler r) { 
		rl.ReportTo(r); 
		dmh=r;
	}	// Default
	public void setChatMessageHandler(RMsgHandler r) { rl.ReportTo(r); }	// Chats
	public void setPrivateMessageHandler(RMsgHandler r) { 
		rl.ReportPrivateTo(r); 
		sph=r;
	} // Private
	public void setWallMessageHandler(RMsgHandler r) { rl.ReportWallTo(r); }	// Walls
	public void setSystemMessageHandler(RMsgHandler r) { 
		rl.ReportSystemTo(r); 
		smh=r;
	}	// System..

	public void setAutoMessageHandler(RMsgHandler r){}
	public void setClientMessageHandler(RMsgHandler r){}

	public void setUserListHandler(wtalk.IUserList ul){} // just here for looks

	//
	// Uncooked functions
	//
	public void sendRaw(String msg) { out.println(msg); }


	//
	// Authentication/Login
	//
	public void connect() throws IOException {
		UsaInit();
		resetReader();

		us = new Socket(mServer, mPort==0? DEFAULT_PORT : mPort);
		out = new PrintWriter(us.getOutputStream(), true);
		rl.ListenTo(new BufferedReader( new InputStreamReader(us.getInputStream())));
		rl.start();
	}

	public void connect(String login) throws IOException {
		connect(mServer, mPort, login);
	}

	public void connect(String server, int port) throws IOException {
		mServer = server;	mPort = port;
		connect();
	}

	public void connect(String server, int port, String login) throws IOException {
		mName = login;
		connect(server, port);
		// send login;
	}

	public void disconnect(String msg) { sendRaw("quit " + msg); }


	public boolean isConnected() { return us!=null && us.isConnected(); }

	public void setName(String name) { sendRaw("nick " + name); }
	public String getName() { return mName; }

	public void setPassword(String passwd) {  }
	public void changePassword(String oldpwd, String newpwd) {  }

	public void setServer(String server) { mServer = server; }
	public String getServer() { return mServer; }
	public void setServerPort(int port) { mPort = port;}
	public int getServerPort() { return mPort; }



	//
	// IM functions
	//
	public void sendMessage(String to, String data) {
		sendRaw("msg " + to + " " + data);

		if(!(to.charAt(0)=='&' || to.charAt(0)=='#'))
			if(sph!=null)
				sph.handle(to, null, data);
			else if(dmh!=null)
				dmh.handle(to, null, data);
	}

	public void whois(String user) { sendRaw("whois " + user); }

	public void addBuddy(String buddy, String group){}
	public void removeBuddy(String buddy, String group){}
	//listBuddies();

	public void setAway(String message){}

	//addDeny();
	//removeDeny();


	//
	// Chat functions
	//
	public void joinChat(String room) { sendRaw("join " + room); }
	public void partChat(String room) { sendRaw("part " + room); }

	public void chatMessage(String room, String data) { sendMessage(room, data); }
	public void setTopic(String room, String topic) {
		if(topic==null || topic.equals(""))
			sendRaw("topic " + room); 
		else
			sendRaw("topic " + room + " " + topic);

	}

	public void chatList(String room) {
		if(room==null || room.equals(""))
			sendRaw("list"); 
		else
			sendRaw("list " + room);
	}


	//
	// Admin functions
	//
	public void sendWall(String msg) {	sendRaw("wall " + msg); }
	public void sendSub(String to, String data){};

	public void setBan(String user, boolean banned){}
}
